const DEBUG = 10;
var showNamespaces = false;
var xmlNamespaces = {};

var RDF = Components
    .classes['@mozilla.org/rdf/rdf-service;1']
    .getService()
    .QueryInterface(Components.interfaces.nsIRDFService);

var RDFC = Components
    .classes["@mozilla.org/rdf/container-utils;1"]
    .getService(Components.interfaces.nsIRDFContainerUtils);

function RDFInspector(aDataSource) {
    this.open(aDataSource);
}
RDFInspector.prototype = {
    mDS      : null,  // Stores the datasource itself
    mURI     : null,  // Shortcut to the datasource URI; read-only
    mDSType  : null,  // Shortcut to the datasource type; read-only
    mLabel   : null,  // User-settable label for this datasource; read/write
    mOpen    : false, // Flag indicating if this datasource is open; read-only
    mPaused  : false, // Flag indicating we are watching this datasource; read/write
    mLoading : false, // Flag indicating this datasource is still loading; read-only

    // Placeholders for internal objects
    mRDFObserver : null,
    mXMLSink     : null,

    // Storage structures for statements
    mStatements : null,

    get label() { return this.mLabel },
    set label(aLabel) { this.mLabel = aLabel },

    get URI() { return this.mURI },
    get type() { return this.mDSType },
    get length() { return this.mStatements.length },

    get isOpen() { return this.mOpen },

    get isPaused() { return this.mPaused },
    set isPaused(aPaused) {
        this.mPaused = aPaused;
    },

    close : function() {
        if (DEBUG > 1) dump('* close()\n');
        this.mOpen = false;
        if (this.mRDFObserver) {
            this.mDS.RemoveObserver(this.mRDFObserver);
            this.mRDFObserver = null;
        }
        if (this.mXMLSink) {
            this.mDS.QueryInterface(Components.interfaces.nsIRDFXMLSink);
            this.mDS.removeXMLSinkObserver(this.mXMLSink);
            this.mDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
            this.mXMLSink = null;
        }
        this.mDS = null;
        this.mStatements = null;
    },

    open : function(aDataSource) {
        if (DEBUG > 1) dump('= RDFInspector.open()\n');
        if (!aDataSource instanceof Components.interfaces.nsIRDFDataSource) {
            var uri = aDataSource;
            if (DEBUG > 2) dump('  + uri = "' + uri + '"\n');
            aDataSource = RDF.GetDataSource(uri);
        }
        if (DEBUG > 2) dump('  + aDataSource = "' + aDataSource + '"\n');

        // Pre-load the datasource metadata
        this.mDS = aDataSource;
        this.mURI = aDataSource.URI;
        if (aDataSource instanceof Components.interfaces.nsIRDFCompositeDataSource)
            this.mDSType = "nsIRDFCompositeDataSource";
        else if (aDataSource instanceof Components.interfaces.nsIRDFRemoteDataSource)
            this.mDSType = "nsIRDFRemoteDataSource";
        else if (aDataSource instanceof Components.interfaces.nsIRDFInMemoryDataSource)
            this.mDSType = "nsIRDFInMemoryDataSource";
        else if (aDataSource instanceof Components.interfaces.nsIRDFPropagatableDataSource)
            this.mDSType = "nsIRDFPropagatableDataSource";
        else if (aDataSource instanceof Components.interfaces.nsIRDFPurgeableDataSource)
            this.mDSType = "nsIRDFPurgeableDataSource";
        else if (aDataSource instanceof Components.interfaces.nsIRDFDataSource)
            this.mDSType = "nsIRDFDataSource";
        else
            this.mDSType = "Unknown";

        if (DEBUG > 2) dump('  + this.mDSType = "' + this.mDSType + '"\n');

        // Initialize our statement storage array
        this.mStatements = new Array();
        if (DEBUG > 2) dump('  + this.mStatements = "' + this.mStatements + '"\n');

        // Add the RDF statement observer
        var parent = this;
        this.mRDFObserver = {
            onAssert : function(aDataSource, aSource, aProperty, aTarget) {
                if (DEBUG > 1) dump('+ RDFInspector.mRDFObserver.onAssert("' + aDataSource + '", "' + aSource.Value + '", "' + aProperty.Value + '", "' + aTarget + '")\n');
                parent.addStatement(aSource, aProperty, aTarget);
                if (DEBUG > 3) dump('  * Added the statement\n');
            },

            onUnassert : function(aDataSource, aSource, aProperty, aTarget) {
                if (DEBUG > 1) dump('+ RDFInspector.mRDFObserver.onUnassert("' + aDataSource + '", "' + aSource.Value + '", "' + aProperty.Value + '", "' + aTarget + '")\n');
            },

            onChange : function(aDataSource, aSource, aProperty, aOldTarget, aNewTarget) {
                if (DEBUG > 1) dump('+ RDFInspector.mRDFObserver.onChange("' + aDataSource + '", "' + aSource.Value + '", "' + aProperty.Value + '", "' + aOldTarget + '", "' + aNewTarget + '")\n');
            },
            onMove : function(aDataSource, aOldSource, aNewSource, aProperty, aTarget) {
                if (DEBUG > 1) dump('+ RDFInspector.mRDFObserver.onMove("' + aDataSource + '", "' + aOldSource.Value + '", "' + aNewSource.Value + '", "' + aProperty.Value + '", "' + aTarget + '")\n');
            },
            onBeginUpdateBatch : function(aDataSource) {
                if (DEBUG > 1) dump('+ RDFInspector.mRDFObserver.onBeginUpdateBatch()\n');
            },
            onEndUpdateBatch : function(aDataSource) {
                if (DEBUG > 1) dump('+ RDFInspector.mRDFObserver.onEndUpdateBatch()\n');
            }
        };
        aDataSource.AddObserver(this.mRDFObserver);

        // Create the XML Sink to watch the file load
        if (aDataSource instanceof Components.interfaces.nsIRDFRemoteDataSource) {
            if (DEBUG > 3) dump('  * This is a remote datasource\n');
            if (!aDataSource.loaded && aDataSource instanceof Components.interfaces.nsIRDFXMLSink) {
                if (DEBUG > 3) dump('  * Adding an XML sink to this datasource\n');
                if (DEBUG > 4) dump('  * 1\n');
                this.mXMLSink = {
                    onBeginLoad : function(sink) {
                        if (DEBUG > 1) dump('+ RDFInspector.mXMLSink.onBeginLoad("' + sink + '")\n');
                        parent.mLoading = true;
                        if (DEBUG > 2) dump('  + parent.mLoading = "' + parent.mLoading + '"\n');
                    },
                    onInterrupt : function(sink) {
                        if (DEBUG > 1) dump('+ RDFInspector.mXMLSink.onInterrupt("' + sink + '")\n');
                    },
                    onResume : function(sink) {
                        if (DEBUG > 1) dump('+ RDFInspector.mXMLSink.onResume("' + sink + '")\n');
                    },
                    onError : function(sink,status,msg) {
                        if (DEBUG > 1) dump('+ RDFInspector.mXMLSink.onError("' + sink + '", "' + status + '", "' + msg + '")\n');
                    },
                    onEndLoad : function(sink) {
                        if (DEBUG > 1) dump('+ RDFInspector.mXMLSink.onEndLoad("' + sink + '")\n');
                        sink.removeXMLSinkObserver(this);
                        sink.QueryInterface(Components.interfaces.nsIRDFDataSource);

                        parent.mLoading = false;
                        if (DEBUG > 2) dump('  + parent.mLoading = "' + parent.mLoading + '"\n');

                        parent.mXMLSink = null;
                    }
                };
                if (DEBUG > 4) dump('  * 2\n');
                aDataSource.QueryInterface(Components.interfaces.nsIRDFXMLSink);
                if (DEBUG > 4) dump('  * 3\n');
                aDataSource.addXMLSinkObserver(this.mXMLSink);
                if (DEBUG > 4) dump('  * 4\n');
            } else {
                if (DEBUG > 3) dump('  * Fetching the statements manually\n');
                this.importStatements();
                if (DEBUG > 4) dump('  * Done fetching statements\n');
            }
        }

        // Suck up the contents if we have a composite datasource
        else if (aDataSource instanceof Components.interfaces.nsIRDFCompositeDataSource) {
            var dsEnum = aDataSource.GetDataSources();
            if (DEBUG > 2) dump('  + dsEnum = "' + dsEnum + '"\n');
            while (dsEnum.hasMoreElements()) {
                var ds = dsEnum.getNext();
                if (DEBUG > 2) dump('  + ds = "' + ds + '"\n');
                if (ds instanceof Components.interfaces.nsIRDFDataSource) {
                    if (DEBUG > 2) dump('  + ds = "' + ds + '"\n');
                    if (DEBUG > 3) dump('  * Fetching the statements manually\n');
                    try {
                        this.importStatements(ds);
                    } catch (e) {
                        if (DEBUG > 0) dump('  Exception occurred while importing statements: ' + e + '\n');
                    }
                    if (DEBUG > 4) dump('  * Done fetching statements\n');
                }
            }
        }

        // Suck up the contents if we have something else
        else {
            if (DEBUG > 3) dump('  * Fetching the statements manually\n');
            this.importStatements();
            if (DEBUG > 4) dump('  * Done fetching statements\n');
        }
    },

    getStatementByIndex : function(aIndex) {
        //if (DEBUG > 1) dump('+ RDFInspector.getStatementByIndex("' + aIndex + '")\n');
        return this.mStatements[aIndex];
    },

    addStatement : function(aSource, aProperty, aTarget) {
        if (DEBUG > 1) dump('+ RDFInspector.addStatement("' + aSource.Value + '", "' + aProperty.Value + '", "' + aTarget + '")\n');
        var element = {
            source   : aSource,
            property : aProperty,
            target   : aTarget,
        };
        this.mStatements.push(element);
        if (DEBUG > 3) dump('  * Statements list is now ' + this.mStatements.length + ' elements long\n');
    },

    importStatements : function(ds) {
        if (DEBUG > 1) dump('= RDFInspector.importStatements("' + ds + '")\n');
        if (!ds) {
            ds = this.mDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
        }
        if (DEBUG > 2) dump('  + ds = "' + ds + '"\n');

        var resourceEnum = ds.GetAllResources();
        if (DEBUG > 2) dump('  + resourceEnum = "' + resourceEnum + '"\n');
        while (resourceEnum.hasMoreElements()) {
            var resource = resourceEnum.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
            if (DEBUG > 2) dump('  + resource.Value = "' + resource.Value + '"\n');

            var predicateEnum = ds.ArcLabelsOut(resource);
            if (DEBUG > 2) dump('  + predicateEnum = "' + predicateEnum + '"\n');
            while (predicateEnum.hasMoreElements()) {
                var predicate = predicateEnum.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
                if (DEBUG > 2) dump('  + predicate.Value = "' + predicate.Value + '"\n');

                var objectEnum;
                try {
                    objectEnum = ds.GetTargets(resource, predicate, true);
                } catch (e) {
                    if (DEBUG > 3) dump('  * Caught an exception while getting predicate targets: ' + e + '.\n');
                }
                if (DEBUG > 2) dump('  + objectEnum = "' + objectEnum + '"\n');
                while (objectEnum.hasMoreElements()) {
                    var object;
                    try {
                        object = objectEnum.getNext();
                    } catch (e) {
                        if (DEBUG > 3) dump('  * Caught an exception while processing the object enumerator: ' + e + '.\n');
                    }

                    // Add the statement count for this datasource
                    object = this.deriveNodeType(object);
                    if (DEBUG > 2) dump('  + object = "' + object + '"\n');
                    this.addStatement(resource, predicate, object);
                }
            }
        }
        if (DEBUG > 3) dump('  * Finished processing resources\n');
    },

    deriveNodeType : function(node) {
        if (node instanceof Components.interfaces.nsIRDFResource) {
            node = node.QueryInterface(Components.interfaces.nsIRDFResource);
        } else if (node instanceof Components.interfaces.nsIRDFLiteral) {
            node = node.QueryInterface(Components.interfaces.nsIRDFLiteral);
        } else if (node instanceof Components.interfaces.nsIRDFDate) {
            node = node.QueryInterface(Components.interfaces.nsIRDFDate);
        } else if (node instanceof Components.interfaces.nsIRDFInt) {
            node = node.QueryInterface(Components.interfaces.nsIRDFInt);
        } else if (node instanceof Components.interfaces.nsIRDFBlob) {
            node = node.QueryInterface(Components.interfaces.nsIRDFBlob);
        } else {
            node = node.QueryInterface(Components.interfaces.nsIRDFNode);
        }
        return node;
    },
};

function RDFInspectorCollection() {
    this.mInspectors = new Array();
    this.mObservers = new Array();
}
RDFInspectorCollection.prototype = {
    mInspectors : null,
    mObservers : null,
    get length() { return this.mInspectors.length },

    AddObserver : function(aObserver) {
        if (DEBUG > 1) dump('= RDFInspectorCollection.AddObserver("' + aObserver + '")\n');
        this.mObservers.push(aObserver);
    },

    addInspector : function(aInspector) {
        if (DEBUG > 1) dump('= RDFInspectorCollection.addInspector("' + aInspector + '")\n');
        this.mInspectors.push(aInspector);
        for (var idx in this.mObservers) {
            try {
                this.mObservers[idx].onAddInspector(aInspector);
            } catch (e) {}
        }
    },

    removeInspector : function(aInspector) {
        if (DEBUG > 1) dump('= RDFInspectorCollection.removeInspector("' + aInspector + '")\n');
        for (var i = 0; i < this.mInspectors.length; i++) {
            if (aInspector == this.mInspectors[i]) {
                if (DEBUG > 4) dump('  * Removing index ' + i + '\n');
                var result = this.mInspectors.slice(i, 1);
                for (var idx in this.mObservers) {
                    try {
                        this.mObservers[idx].onRemoveInspector(aInspector);
                    } catch (e) {}
                }
                return result[0];
            }
        }
        return null;
    },

    removeItem : function(aIndex) {
        if (DEBUG > 1) dump('= RDFInspectorCollection.removeItem("' + aIndex + '")\n');
        var result = this.mInspectors.slice(aIndex, 1);
        for (var idx in this.mObservers) {
            try {
                this.mObservers[idx].onRemoveInsector(result[0]);
            } catch (e) {}
        }
        return result[0];
    },

    item : function(aIndex) {
        //if (DEBUG > 1) dump('= RDFInspectorCollection.item("' + aIndex + '")\n');
        return this.mInspectors[aIndex];
    },
};

function RDFInspectorTreeView() {}
RDFInspectorTreeView.prototype = {
    // default values
    mRowCount: null,
    treeBox: null,

    get wrappedJSObject() { return this; },

    // nsISupports
    QueryInterface : function(aIID) {
        if (Components.interfaces.nsIClassInfo.equals(aIID))
            return RDFInspectorTreeView.prototype;
        if (Components.interfaces.nsITreeView.equals(aIID) ||
                Components.interfaces.nsISupportsWeakReference.equals(aIID) ||
                Components.interfaces.nsISupports.equals(aIID))
            return this;
        throw 0x80004002; // Components.results.NS_NOINTERFACE;
    },

    // nsIClassInfo
    getInterfaces: function(count) {
        count.value = 4;
        return [
            Components.interfaces.nsITreeView,
            Components.interfaces.nsIClassInfo, 
            Components.interfaces.nsISupportsWeakReference, 
            Components.interfaces.nsISupports
        ];
    },
    getHelperForLanguage: function(language) { return null; },
    get contractID() { return null; },
    get classDescription() { return "nsTreeView"; },
    get classID() { return null; },
    get implementationLanguage() { return Components.interfaces.nsIProgrammimgLanguage.JAVASCRIPT; },
    get flags() { return Components.interfaces.nsIClassInfo.DOM_OBJECT; },

    // nsITreeView
    get rowCount() {
        if (DEBUG > 1) dump('= get RDFInspectorTreeView.rowCount()\n');
        return this.mRowCount;
    },
    set rowCount(aCount) {
        if (DEBUG > 1) dump('= set RDFInspectorTreeView.rowCount("' + aCount + '")\n');
        if (DEBUG > 3) dump('  * 1\n');
        this.mRowCount = aCount;
        if (DEBUG > 3) dump('  * 2\n');
        if (this.treeBox) {
            try {
                this.treeBox.rowCountChanged(1, aCount);
                if (DEBUG > 3) dump('  * Updated row count\n');
            } catch (e) {
                if (DEBUG > 0) dump('  Exception occurred while updating the tree box row count: ' + e + '\n');
            }
        }
    },

    selection: null,
    getRowProperties: function(row, prop) { },
    getCellProperties: function(row, column, prop) { },
    getColumnProperties: function(column, elem, prop) { },
    isContainer: function(row) { },
    isContainerOpen: function(row) { },
    isContainerEmpty: function(row) { return false; },
    isSeparator: function(row) { return false; },
    isSorted: function() { return false; },
    // d&d not implemented yet!
    canDropOn: function(row) { return false; },
    canDropBeforeAfter: function(row, before) { return false; },
    drop: function(row, orientation) { },
    getParentIndex: function(row) { },
    hasNextSibling: function(row, after) { },
    getLevel: function(row) { },
    getImageSrc: function(row, column) { },
    getProgressMode : function(row,column) { },
    getCellValue: function(row, column) { },
    getCellText: function(row, column) { }, // MUST Override!
    setTree: function(treeBox) { this.treeBox = treeBox; if (!treeBox) this.selection = null; },
    cycleHeader: function(col, elem) { },
    selectionChanged: function() { },
    cycleCell: function(row, column) { },
    isEditable: function(row, column) { return false; },
    performAction: function(action) { },
    performActionOnCell: function(action, row, column) { },
    toggleOpenState: function(row) { },
};

function RDFInspectorDataSourceTreeView(aCollection) {
    if (DEBUG > 1) dump('= RDFInspectorDataSourceTreeView("' + aCollection + '")\n');
    this.mCollection = aCollection;
    aCollection.AddObserver(this);
    this.rowCount = 0;
}
RDFInspectorDataSourceTreeView.prototype = new RDFInspectorTreeView();
RDFInspectorDataSourceTreeView.prototype.getCellText = function(row, column) {
    //if (DEBUG > 1) dump('= RDFInspectorDataSourceTreeView.getCellText("' + row + '", "' + column + '")\n');
    var dsItem = this.mCollection.item(row);

    var text = null;
    // Watch column
    if (column == 'watchDSCol')
        text = '';

    // Label column
    else if (column == 'labelDSCol')
        text = dsItem.label;

    // URI column
    else if (column == 'uriDSCol')
        text = dsItem.URI;

    // Type column
    else if (column == 'typeDSCol')
        text = dsItem.type;

    // Size column
    else if (column == 'sizeDSCol')
        text = dsItem.length;

    return text;
};

RDFInspectorDataSourceTreeView.prototype.onAddInspector = function(aInspector) {
    if (DEBUG > 1) dump('= RDFInspectorDataSourceTreeView.onAddInspector ("' + aInspector + '")\n');
    this.rowCount++;
    if (DEBUG > 2) dump('  + this.rowCount = "' + this.rowCount + '"\n');
    this.treeBox.invalidate();
    if (DEBUG > 3) dump('  * Invalidated treeBox\n');
};

RDFInspectorDataSourceTreeView.prototype.onRemoveInspector = function(aInspector) {
    if (DEBUG > 1) dump('= RDFInspectorDataSourceTreeView.onRemoveInspector ("' + aInspector + '")\n');
    this.rowCount--;
    if (DEBUG > 2) dump('  + this.rowCount = "' + this.rowCount + '"\n');
    this.treeBox.invalidate();
    if (DEBUG > 3) dump('  * Invalidated treeBox\n');
};

function RDFInspectorStatementTreeView(aDS) {
    if (DEBUG > 1) dump('= RDFInspectorStatementTreeView("' + aDS + '")\n');
    this.mDS = aDS;
    this.rowCount = aDS.length;
}
RDFInspectorStatementTreeView.prototype = new RDFInspectorTreeView();
RDFInspectorStatementTreeView.prototype.getCellText = function(row, column) {
    //if (DEBUG > 1) dump('= RDFInspectorStatementTreeView.getCellText("' + row + '", "' + column + '")\n');
    var statement = this.mDS.getStatementByIndex(row);
    //if (DEBUG > 2) dump('  + statement = "' + statement + '"\n');

    var text = null;
    if (column == 'numStmtCol') {
        text = row + 1;
    }

    else if (column == 'subjectStmtCol') {
        text = statement.source.Value;
        if (showNamespaces)
            text = convertURIToNS(text);
    }

    else if (column == 'predicateStmtCol') {
        text = statement.property.Value;
        if (showNamespaces)
            text = convertURIToNS(text);
    }

    else if (column == 'objectStmtCol') {
        text = statement.target.Value;
        if (statement.target instanceof Components.interfaces.nsIRDFResource && showNamespaces)
            text = convertURIToNS(text);
    }

    //if (DEBUG > 2) dump('  + text = "' + text + '"\n');
    return text;
};

function convertURIToNS(aURI) {
    for (var idx = 0; idx < xmlNamespaces.length; idx++) {
        if (!xmlNamespaces[idx].enabled) continue;
        if (aURI.indexOf(xmlNamespaces[idx].uri) == 0) {
            return xmlNamespaces[idx].prefix + ":" + aURI.substring(xmlNamespaces[idx].uri.length);
        }
    }
    return aURI;
}

window.updateNamespaceList = updateNamespaceList;
function updateNamespaceList() {
    xmlNamespaces = RDFInspectorSettings.GetAllNamespaces();
    gStatementTree.treeBoxObject.invalidate();
}

var gRDFCollection = new RDFInspectorCollection();
var gDatasourceTree, gStatementTree;

function onLoad() {
    if (DEBUG > 1) dump('= onLoad()\n');
    // Set the datasources up
    if (DEBUG > 4) dump("  * onLoad() : 1\n");
    gDatasourceTree = document.getElementById("datasourceTree");
    gStatementTree = document.getElementById("statementListTree");
    if (DEBUG > 4) dump("  * onLoad() : 2\n");
    updateNamespaceList();

    if (window.arguments && window.arguments.length) {
        for (var i = 0; i < window.arguments.length; i++) {
            loadDataSource(window.arguments[i]);
        }
    }
    if (DEBUG > 4) dump("  * onLoad() : 3\n");
    var dsTreeView = new RDFInspectorDataSourceTreeView(gRDFCollection);
    if (DEBUG > 2) dump('  + dsTreeView = "' + dsTreeView + '"\n');
    if (DEBUG > 2) dump('  + dsTreeView.rowCount = "' + dsTreeView.rowCount + '"\n');
    gDatasourceTree.view = dsTreeView;
    if (DEBUG > 4) dump("  * onLoad() : 4\n");
}

window.loadDataSource = loadDataSource;
function loadDataSource(uri) {
    if (DEBUG > 1) dump('= loadDataSource("' + uri + '")\n');
    var ds;

    // Load the datasource
    if (uri instanceof Components.interfaces.nsIRDFDataSource) {
        ds = uri;
    } else {
        ds = RDF.GetDataSource(uri);
    }
    if (DEBUG > 2) dump('  + ds = "' + ds + '"\n');

    var dsObj = new RDFInspector(ds);
    if (DEBUG > 2) dump('  + dsObj = "' + dsObj + '"\n');

    gRDFCollection.addInspector(dsObj);
    if (DEBUG > 2) dump('  + gRDFCollection.length = "' + gRDFCollection.length + '"\n');
}

// EVENT HANDLERS
function openURI() {
    if (DEBUG > 1) dump('= openURI()\n');
    openDialog("chrome://rdfinspector/content/openURI.xul", "_blank", "chrome,modal,titlebar", window);
}

function newDataSource() {
    alert("This doesn't work yet; sorry.");
}

function saveAs() {
    alert("This doesn't work yet; sorry.");
}

function closeDS() {
    if (DEBUG > 1) dump('= closeDS()\n');
    alert("This doesn't work yet; sorry.");
    return;
    var idx = gDatasourceTree.view.selection.currentIndex;
    if (DEBUG > 2) dump('  + idx = "' + idx + '"\n');
    gRDFCollection.removeItem(idx);
    if (DEBUG > 3) dump('  * Done closing\n');
}

function openFile() {
    if (DEBUG > 1) dump('= openFile()\n');
    var str = document.getElementById("rdfinspectorStrings");
    var fp = Components
        .classes["@mozilla.org/filepicker;1"]
        .createInstance(Components.interfaces.nsIFilePicker);

    fp.init(window, str.getString("openFileTitle"), Components.interfaces.nsIFilePicker.modeOpen);
    fp.appendFilter(str.getString("rdfFiles"), "*.rdf");
    fp.appendFilters(Components.interfaces.nsIFilePicker.filterXML);
    fp.appendFilters(Components.interfaces.nsIFilePicker.filterAll);

    var result = fp.show();
    if (result == Components.interfaces.nsIFilePicker.returnOK) {
        var ioServ = Components
            .classes["@mozilla.org/network/io-service;1"]
            .getService(Components.interfaces.nsIIOService);

        var fph = ioServ
            .getProtocolHandler("file")
            .QueryInterface(Components.interfaces.nsIFileProtocolHandler);

        var fileURI = fph.getURLSpecFromFile(fp.file);

        if (DEBUG > 3) dump(" * Before loadDataSource\n");
        loadDataSource(fileURI);
        if (DEBUG > 3) dump(" * After loadDataSource\n");
    }
}

function getSelectedURI(event) {
    if (DEBUG > 1) dump('= getSelectedURI("' + event + '")\n');
    var currentIndex = event.target.view.selection.currentIndex;
    if (currentIndex < 0) return null;
    return event.target.view.getResourceAtIndex(currentIndex).Value;
}

function selectDatasourceTree(event) {
    if (DEBUG > 1) dump('= selectDatasourceTree("' + event + '")\n');
    var tree = event.target;
    var bcastClose = document.getElementById("bcastCloseDS");
    var idx = tree.view.selection.currentIndex;

    if (idx == -1) {
        bcastClose.setAttribute("disabled", "true");
        return;
    } else {
        bcastClose.setAttribute("disabled", "false");
    }

    var stmtTreeView = new RDFInspectorStatementTreeView(gRDFCollection.item(idx));
    if (DEBUG > 2) dump('  + stmtTreeView = "' + stmtTreeView + '"\n');
    if (DEBUG > 2) dump('  + stmtTreeView.rowCount = "' + stmtTreeView.rowCount + '"\n');
    gStatementTree.view = stmtTreeView;
}

function toggleNamespacePrefixes() {
    if (DEBUG > 1) dump('= toggleNamespacePrefixes()\n');
    showNamespaces = !showNamespaces;
    var bcaster = document.getElementById('bcast_tbViewOptions_mnUseNSPrefixes');
    bcaster.setAttribute('checked', showNamespaces.toString());
}

function openEditNamespacesDialog() {
    window.openDialog(
        'chrome://rdfinspector/content/editNamespace.xul',
        'rdfinspector-editNamespace',
        'chrome,width=450,height=400'
    );
}
